# frozen_string_literal: true

require 'dry-initializer'
require 'dry-container'

require 'auth_strategist/version'
require 'auth_strategist/configuration'
require 'auth_strategist/registry'
require 'auth_strategist/errors/unauthorized'
require 'auth_strategist/errors/authorized_not_implemented'
require 'auth_strategist/strategy_interface'
require 'auth_strategist/build_strategy'
require 'auth_strategist/authorization'

module AuthStrategist
  class << self
    def configuration
      @configuration ||= Configuration.new
    end

    def configure
      yield(configuration) if block_given?
      configuration
    end

    def strategies
      yield(Registry) if block_given?
      Registry
    end
  end
end

require 'generators/auth_strategist/install_generator' if defined?(Rails)
