# frozen_string_literal: true

module AuthStrategist
  class BuildStrategy
    def self.call(params = {})
      new.call(params)
    end

    def call(params = {})
      choose_strategy(params)
    end

    private

    attr_reader :strategy

    def choose_strategy(params = {})
      strategy_key = params[:strategy]
      raise(strategy_key_nil) if strategy_key.nil?

      strategy_class = begin
                         strategies.resolve(strategy_key)
                       rescue StandardError
                         nil
                       end

      raise(strategy_not_found(strategy_key)) if strategy_class.nil?

      strategy_params = params.dup.tap { |h| h.delete(:strategy) }
      strategy_class.new(strategy_params)
    end

    def strategies
      AuthStrategist.strategies
    end

    def strategy_not_found(strategy_key)
      StandardError.new("Strategy '#{strategy_key}' was not found.")
    end

    def strategy_key_nil
      StandardError.new(':strategy option must not be nil.')
    end
  end
end
